@@ -93,7 +93,7 @@ typedef struct {
 	char bv[32];
 } UriStrspnSet; 
 
-UriStrspnSet uri_strspn_sets[] = {
+static UriStrspnSet uri_strspn_sets[] = {
 	{":@]" GNOME_VFS_URI_PATH_STR, FALSE, ""},
 	{"@" GNOME_VFS_URI_PATH_STR, FALSE, ""},
 	{":" GNOME_VFS_URI_PATH_STR, FALSE, ""},
@@ -163,7 +163,7 @@ split_toplevel_uri (const gchar *path, guint path_len,
 	success = FALSE;
 
 	if (path == NULL || path_len == 0) {
-		return NULL;
+		return g_strdup ("/");
 	}
 	
 
@@ -358,7 +358,7 @@ set_uri_element (GnomeVFSURI *uri,
 	char *escaped_text;
 
 	if (text == NULL || len == 0) {
-		uri->text = g_strdup("/");;
+		uri->text = g_strdup ("/");
 		return;
 	}
 
@@ -384,7 +384,9 @@ set_uri_element (GnomeVFSURI *uri,
 	 * meaning.
 	 */
 	if ( ! (strcmp (uri->method_string, "http") == 0 
-	        || strcmp (uri->method_string, "eazel-services") == 0
+	        || strcmp (uri->method_string, "https") == 0
+		|| strcmp (uri->method_string, "dav") == 0
+		|| strcmp (uri->method_string, "davs") == 0
 	        || strcmp (uri->method_string, "ghelp") == 0
 	        || strcmp (uri->method_string, "gnome-help") == 0
 	        || strcmp (uri->method_string, "help") == 0
@@ -410,7 +412,12 @@ get_method_string (const gchar *substring, gchar **method_string)
 	     p++)
 		;
 
-	if (*p == ':') {
+	if (*p == ':'
+#ifdef G_OS_WIN32
+	              &&
+	    !(p == substring + 1 && g_ascii_isalpha (*substring))
+#endif
+								 ) {
 		/* Found toplevel method specification.  */
 		method = g_strndup (substring, p - substring);
 		*method_string = g_ascii_strdown (method, -1);
@@ -489,6 +496,38 @@ parse_uri_substring (const gchar *substring, GnomeVFSURI *parent)
 GnomeVFSURI *
 gnome_vfs_uri_new (const gchar *text_uri)
 {
+#ifdef G_OS_WIN32
+	gchar *method_string;
+	const gchar *rest;
+
+	rest = get_method_string (text_uri, &method_string);
+	if (strcmp (method_string, "file") == 0) {
+		/* Use gnome_vfs_escape_path_string() to replace
+		 * backslashes with slashes etc. Be careful not to
+		 * pass the colon after the method string (if
+		 * present), or the colon after a drive letter to
+		 * gnome_vfs_escape_path_string().
+		 */
+		const gchar *past_root = g_path_skip_root (rest);
+		gchar *escaped = gnome_vfs_escape_path_string (past_root);
+		gchar *root = g_strndup (rest, past_root - rest);
+		gchar *full = g_strconcat (method_string,
+					   ":",
+					   root,
+					   escaped,
+					   NULL);
+		GnomeVFSURI *result =
+			gnome_vfs_uri_new_private (full, FALSE, FALSE, TRUE);
+
+		g_free (full);
+		g_free (escaped);
+		g_free (root);
+		g_free (method_string);
+
+		return result;
+	}
+	g_free (method_string);
+#endif
 	return gnome_vfs_uri_new_private (text_uri, FALSE, FALSE, TRUE);
 }
 
@@ -1742,18 +1781,21 @@ gnome_vfs_uri_extract_short_name (const GnomeVFSURI *uri)
 	short_path_name = gnome_vfs_unescape_string (escaped_short_path_name, "/");
 	g_free (escaped_short_path_name);
 
-	host_name = NULL;
 	if (short_path_name != NULL
-		&& strcmp (short_path_name, GNOME_VFS_URI_PATH_STR) == 0) {
-		host_name = gnome_vfs_uri_get_host_name (uri);
-	}
-
-	if (host_name == NULL || strlen (host_name) == 0) {
+		&& strcmp (short_path_name, GNOME_VFS_URI_PATH_STR) != 0) {
 		return short_path_name;
 	}
 
-	g_free (short_path_name);
-	return g_strdup (host_name);
+	host_name = gnome_vfs_uri_get_host_name (uri);
+
+	if (host_name != NULL && strlen (host_name) != 0) {
+		g_free (short_path_name);
+		return g_strdup (host_name);
+	} else if (short_path_name != NULL) {
+		return short_path_name;
+	} else {
+		return g_strdup (gnome_vfs_uri_get_path (uri));
+	}
 }
 
 /**
